#ifndef _COMPILER_H
#define _COMPILER_H

#if defined(__KCC)
#define NO_DEF_ARG_FOR_MEMBER_TEMPLATES
#define COMPILER_CHECKED
#endif

// c++ -E -dM test.cpp
// EGCS: __GNUC__ == 2 && __GNUC_MINOR__ == 90
// g++: __GNUC__ == 2 && __GNUC_MINOR__ == 80
#if defined(__GNUG__) && (__GNUG__ == 2)
#define NO_NAMESPACE
#define NO_ALLOCATOR_MEMBERS
#define NO_STD_ITERATOR
#define NO_MEMBER_TEMPLATES
#define COMPILER_CHECKED
#endif

#if defined(__BCPLUSPLUS__) && (__BCPLUSPLUS__ == 0x530)
//#define BCC53
#define NO_MEMBER_TEMPLATES
#define NO_ALLOCATOR_REBIND  // allocator definiert kein rebind
#define ONLY_3_ARGS_STD_ITERATOR  // iterator<tag, T, ptrdiff_t> ohne T*, T&
#define NO_COUNT_WITH_RETURN
#define NO_ITERATOR_TRAITS
#define NO_STDARG_FOR_OSTREAMITER   // ostream_iterator<int, char>
#define NO_STDARG_FOR_ISTREAMITER   // istream_iterator<int, char>
#define COMPILER_CHECKED
#endif

#if defined(__BCPLUSPLUS__) && (__BCPLUSPLUS__ <= 0x520)
//#define BCC52
#define NO_ALLOCATOR_REBIND  // allocator definiert kein rebind
#define NO_STRINGSTREAMS
#define OLD_ISTREAMITER  // istream_iterator<int, ptrdiff_t>
#define NO_COUNT_WITH_RETURN
#define NO_GLOBAL_OPS_FOR_BITSET
#define NO_STD_ITERATOR  // forward_iterator statt iterator<forward_iterator_tag ...>
#define OLD_REVERSE_ITERATORS
#define NO_ALLOCATOR_MEMBERS
#define NO_MEMBER_TEMPLATES
#define NO_BASIC_STREAM
#define OLD_TEMPLATE_SYNTAX  // template<> geht nicht
#define NO_TEMPLATE_SPECIAL
#define NO_TEMPLATE_STDARG
#define NO_STD_IOSTREAM
#define NO_ITERATOR_TRAITS
#define NO_MULTIPLIES
#define NO_STD_ITERATOR
#define NO_NAMESPACE_REL_OPS
#define COMPILER_CHECKED
#endif

#if defined(_MSC_VER)
//#define CL11
#define NO_ALLOCATOR_REBIND  // allocator definiert kein rebind
#define ONLY_3_ARGS_STD_ITERATOR
#define OLD_REVERSE_ITERATORS
#define NO_MEMBER_TEMPLATES   // Member Templates in der Klasse definiert geht, auerhalb nicht
#define NO_LOCAL_FOR
#define COMPILER_CHECKED
#define NO_ITERATOR_TRAITS
#define NO_MIN
#define NO_MAX
#define OLD_AUTO_PTR
#define BVECTOR   // _Bvector statt vector<bool>
#endif

#if defined(__WATCOM_CPLUSPLUS__) && (__WATCOM_CPLUSPLUS__ == 1100)
#error Watcom 11.0 hat die STL noch nicht implementiert :-(
#endif

//#if defined(__HIGHC__)
//#if defined(__SUNPRO_CC)

#ifndef COMPILER_CHECKED
#error Compiler u.U. noch nicht korrekt in compiler.h eingetragen.
#endif

//-----------------------------------

#ifdef NOBOOL
typedef int bool;
const bool false = 0, true = !false;
#endif

#ifdef NOMUTABLE
#define mutable
#endif

#ifdef NO_LOCAL_FOR
#define for if(0);else for
#endif

#if defined(NO_MULTIPLIES) && !defined(SK_FUNCTIONAL_H)
#define multiplies times
#endif

// #include ist u.U. kritisch...
#ifdef NO_GLOBAL_OPS_FOR_BITSET
#include <bitset>
#endif

// Ab hier folgen Algorithmen:

#ifndef NO_NAMESPACE
namespace std {
#endif

#ifdef NO_FIND_FIRST_OF
template<class ForwardIterator1, class ForwardIterator2>
ForwardIterator1
find_first_of(ForwardIterator1 first1, ForwardIterator1 last1,
							ForwardIterator2 first2, ForwardIterator2 last2) {
	while (first1 != last1) {
    for (ForwardIterator2 i2 = first2; i2 != last2; ++i2)
      if (*first1 == *i2)
				return first1;
    ++first1;
  }
  return first1;
}

template<class ForwardIterator1, class ForwardIterator2,
         class BinaryPredicate>
ForwardIterator1
find_first_of(ForwardIterator1 first1, ForwardIterator1 last1,
              ForwardIterator2 first2, ForwardIterator2 last2,
              BinaryPredicate pred) {
	while (first1 != last1) {
    for (ForwardIterator2 i2 = first2; i2 != last2; ++i2)
      if (pred(*first1, *i2))
				return first1;
    ++first1;
  }
  return first1;
}
#endif

#ifdef NO_FIND_END
template<class ForwardIterator1, class ForwardIterator2>
ForwardIterator1
find_end(ForwardIterator1 first1, ForwardIterator1 last1,
         ForwardIterator2 first2, ForwardIterator2 last2) {
  ForwardIterator1 i = last1;
  while (first1 != last1) {
    // bis last1 - (last2-first2) waere genug
    ForwardIterator1 i1 = first1;
    ForwardIterator2 i2 = first2;
    while (i1 != last1 && i2 != last2 && *i1 == *i2) {
      ++i1;
      ++i2;
    }
    if (i2 == last2)
			i = first1;
    ++first1;
  }
  return i;
}

template<class ForwardIterator1, class ForwardIterator2,
         class BinaryPredicate>
ForwardIterator1
find_end(ForwardIterator1 first1, ForwardIterator1 last1,
         ForwardIterator2 first2, ForwardIterator2 last2,
         BinaryPredicate pred) {
  ForwardIterator1 i = last1;
  while (first1 != last1) {
    ForwardIterator1 i1 = first1;
    ForwardIterator2 i2 = first2;
    while (i1 != last1 && i2 != last2 && pred(*i1, *i2)) {
      ++i1;
      ++i2;
		}
		if (i2 == last2)
			i = first1;
		++first1;
	}
	return i;
}
#endif

#ifdef NO_COUNT_WITH_RETURN
#ifdef NO_ITERATOR_TRAITS
template<class InputIterator, class T>
int  // iterator_trait<InputIterator>::distance_type
count(InputIterator first, InputIterator last, const T& value) {
	int n = 0;
	while (first != last)
		if (*first++ == value)
			++n;
	return n;
}

template<class InputIterator, class Predicate>
int  // iterator_trait<InputIterator>::distance_type
count_if(InputIterator first, InputIterator last, Predicate pred) {
	int n = 0;
	while (first != last)
		if (pred(*first++))
			++n;
	return n;
}
#endif
#endif

#ifdef NO_SEARCH_N
template <class ForwardIterator, class Size, class T>
ForwardIterator
search_n(ForwardIterator first, ForwardIterator last,
				 Size count, const T& value) {
	ForwardIterator i = first;
  Size n = 0;
  while (first != last) {
    if (*first == value) {
      ++n;
      if (n == 1)
				i = first;
      if (n == count)
				return i;
    } else
      n = 0;
    ++first;
  }
  return last;
}

template <class ForwardIterator, class Size, class T,
          class BinaryPredicate>
ForwardIterator
search_n(ForwardIterator first, ForwardIterator last,
         Size count, const T& value, BinaryPredicate pred) {
  ForwardIterator i = first;
  Size n = 0;
  while (first != last) {
    if (pred(*first, value)) {
      ++n;
      if (n == 1)
				i = first;
      if (n == count)
				return i;
    } else
      n = 0;
    ++first;
  }
  return last;
}
*/
#endif

#ifdef NO_GLOBAL_OPS_FOR_BITSET
template<size_t N> inline
ostream& operator<<(ostream& os, const bitset<N>& b)
{ return os << b.to_string(); }
/*
template<size_t N>
istream& operator>>(istream& is, bitset<N>& b)
{ return is; }
*/
template<size_t N> inline
	bitset<N> operator&(const bitset<N>& lhs, const bitset<N>& rhs)
		{ return bitset<N>(lhs) &= rhs; }

template<size_t N> inline
	bitset<N> operator|(const bitset<N>& lhs, const bitset<N>& rhs)
		{ return bitset<N>(lhs) |= rhs; }

template<size_t N> inline
	bitset<N> operator^(const bitset<N>& lhs, const bitset<N>& rhs)
		{ return bitset<N>(lhs) ^= rhs; }
#endif

#ifdef NO_MIN
template<class T> inline T min(T x, T y) { return x < y ? x : y; }
template<class T, class Comp> inline T min(T x, T y, Comp cmp) { return cmp(x, y) ? x : y; }
#endif

#ifdef NO_MAX
template<class T> inline T max(T x, T y) { return y < x ? x : y; }
template<class T, class Comp> inline T max(T x, T y, Comp cmp) { return cmp(y, x) ? x : y; }
#endif

#ifndef NO_NAMESPACE
}  // namespace std
#endif

#endif

