#include <algorithm>
#include <functional>
#include <iostream>
#include <iterator>
using namespace std;

// Vier Hilfsfunktionen find_end, die nach einem Wert oder mit Prdikat
// suchen, jeweils fr Forward- und Bidirectional-Iteratoren.

template<class ForwardIterator, class T>
ForwardIterator
find_end(ForwardIterator first, ForwardIterator last, const T& value,
	forward_iterator_tag)
{
	ForwardIterator i(last);
	while (first != last) {
		if (*first == value)
			i = first;
		++first;
	}
	return i;
}

template<class BidirectionalIterator, class T>
BidirectionalIterator
find_end(BidirectionalIterator first, BidirectionalIterator last,
	const T& value, bidirectional_iterator_tag)
{
	if (first == last)
		return last;
	BidirectionalIterator i(last);
	do {
		if (*--i == value)
			return i;
	} while (i != first);
	return last;
}

template<class ForwardIterator, class Predicate>
ForwardIterator
find_end_if(ForwardIterator first, ForwardIterator last, Predicate pred,
	forward_iterator_tag)
{
	ForwardIterator i(last);
	while (first != last) {
		if (pred(*first))
			i = first;
		++first;
	}
	return i;
}

template<class BidirectionalIterator, class Predicate>
BidirectionalIterator
find_end_if(BidirectionalIterator first, BidirectionalIterator last,
	Predicate pred, bidirectional_iterator_tag)
{
	if (first == last)
		return last;
	BidirectionalIterator i(last);
	do {
		if (pred(*--i))
			return i;
	} while (i != first);
	return last;
}

// Die beiden Funktionen, die der Benutzer aufruft:

template<class ForwardIterator, class T> inline
ForwardIterator
find_end(ForwardIterator first, ForwardIterator last, const T& value) {
	return find_end(first, last, value, iterator_traits<ForwardIterator>::iterator_category());
}

template<class ForwardIterator, class Predicate> inline
ForwardIterator
find_end_if(ForwardIterator first, ForwardIterator last, Predicate pred) {
	return find_end_if(first, last, pred, iterator_traits<ForwardIterator>::iterator_category());
}

// Testprogramm

int main() {
	const int f[] = { 2, 3, 1, 4, 3, 1, 5 };
	const int* i = find_end(f, f + 7, 3);
	if (i != f + 7)
		cout << "3 gefunden, Index: " << (i - f) << endl;
	i = find_end_if(f, f + 7, bind2nd(less<int>(), 4));
	if (i != f + 7)
		cout << "letzte Zahl kleiner 4 hat den Index: " << (i - f) << endl;
}
