#include <algorithm>
#include <iostream>
#include <iterator>
#include <typeinfo>
#include <vector>
#include "../../einsatz/aktie.h"
#include "../../einsatz/anleihe.h"
#include "../../einsatz/wertpapier.h"
#include "../../einsatz/wp_objekte.h"
using namespace std;

template<class Typ, class Iter>
class TypIter : public iterator<forward_iterator_tag, Typ> {
public:
	TypIter(Iter first, Iter last) : cur(first), end(last)
		{ if (cur != end && !ok()) next(); }
	explicit TypIter(Iter last) : cur(last), end(last) { }
	TypIter& operator++() { next(); return *this; }
	TypIter operator++(int) { TypIter tmp(*this); next(); return tmp; }
	Typ& operator*() const { return dynamic_cast<Typ&>(**cur); }
	Typ* operator->() const { return &**this; }
	bool operator==(const TypIter& x) const { return cur == x.cur; }
	bool operator!=(const TypIter& x) const { return cur != x.cur; }
	TypIter() { }    // um Anforderungen an Forward-Iteratoren zu erfllen
	// Destruktor, Copy-Konstruktor und Zuweisungsoperator werden generiert
private:
	bool ok() const { return typeid(**cur) == typeid(Typ); }
	void next() { do ++cur; while(cur != end && !ok()); }
	Iter cur, end;    // end ist nur wegen operator= nicht const
};

int main() {
	typedef vector<Wertpapier*> WPVec;
	WPVec v(wp, wp + n);
	TypIter<Aktie, WPVec::iterator> a(v.begin(), v.end()), b(v.end());
	cout << "Aktien:\n";
	copy(a, b, ostream_iterator<Aktie>(cout, "\n"));
	cout << "\nAnleihen:\n";
	copy(TypIter<Anleihe, WPVec::iterator>(v.begin(), v.end()),
		TypIter<Anleihe, WPVec::iterator>(v.end()),
		ostream_iterator<Anleihe>(cout, "\n"));
}
