#ifndef _BINBAUM_H
#define _BINBAUM_H

#include <iostream>
#include <iomanip>

template<class T> class BinBaum;
template<class T> std::ostream& operator<<(std::ostream&, const BinBaum<T>&);

template<class T>
class BinBaum {
	struct Element {
		Element(const T& s, Element* v, Element* l = 0, Element* r = 0)
			: schluessel(s), vater(v), links(l), rechts(r) { }
		T schluessel;
		Element* vater;  // wird fr den Weg nach "oben" bentigt
		Element* links, *rechts;
	} *wurzel;
public:
	BinBaum() : wurzel(0) { }
	class Const_Iter {
	public:
		// Const_Iter(Iter i) : c(i.c) { }
		Const_Iter(const Element* cc) : c(cc) { }
		bool operator==(Const_Iter i) const { return c == i.c; }
		bool operator!=(Const_Iter i) const { return c != i.c; }
		Const_Iter& operator++() {
			c = naechster(c);
			return *this;
		}
		Const_Iter operator++(int) {
			Const_Iter tmp(*this);
			operator++();
			return tmp;
		}
		const T& operator*() const { return c->schluessel; }
	private:
		const Element* c;    // Cursor auf das aktuelle Element
	};
	friend class Const_Iter;
	Const_Iter start() const { return erster(); }
	Const_Iter ende() const { return 0; }
	Const_Iter suchen(const T& x) const { return suche(x); }
	// ...

	~BinBaum();
	void neuesBlatt(const T&);
	class Iter;
	friend class Iter;
	class Iter {
	public:
		Iter(Element* tt) : c(tt) { }
		bool operator==(const Iter& i) const { return c == i.c; }
		bool operator!=(const Iter& i) const { return c != i.c; }
		Iter& operator++() {
			c = const_cast<Element*>(naechster(c));
			return *this;
		}
		Iter operator++(int) {
			Iter tmp(*this);
			operator++();
			return tmp;
		}
		T& operator*() const { return c->schluessel; }
	private:
		friend class Const_Iter;
		Element* c;
	};
	Iter start();
	Iter ende();
	Iter suchen(const T& x);
	friend std::ostream& operator<<<>(std::ostream&, const BinBaum&);
	// Falls das nicht bersetzt wird, 
	// die Template-Kennzeichnung <> hinter dem Funktionsnamen operator<< lschen.
private:
	void loesche(Element* wurzel);
	void neuesBlatt(Element*&, const T&, Element* = 0);
	void drucke(std::ostream&, const Element*, int) const;
	std::ostream& drucke(std::ostream&) const;

private:
	static const Element* naechster(const Element*);
	const Element* erster() const;
	const Element* suche(const T&) const;
};

template<class T>
const typename BinBaum<T>::Element* BinBaum<T>::naechster(const Element* z) {
	if (z->rechts != 0) {
		z = z->rechts;
		while (z->links != 0)
			z = z->links;
	} else if (z->vater->links == z)
		z = z->vater;
	else {
		while (z->vater != 0 && z->vater->rechts == z)
			z = z->vater;
		z = z->vater;
	}
	return z;
}

template<class T>
const typename BinBaum<T>::Element* BinBaum<T>::erster() const {
	const Element* z = wurzel;
	while (z->links != 0)
		z = z->links;
	return z;
}

template<class T>
const typename BinBaum<T>::Element* BinBaum<T>::suche(const T& x) const {
	const Element* z = wurzel;
	while (z != 0) {
		if (x < z->schluessel)
			z = z->links;
		else if (z->schluessel < x)
			z = z->rechts;
		else    // weder kleiner noch grer, dann gefunden
			return z;
	}
	return 0;    // nicht gefunden
}

//------------------------------------------------------------------------

template<class T>
void BinBaum<T>::loesche(Element* z) {
	if (z) {
		loesche(z->links);
		loesche(z->rechts);
		delete z;
	}
}

template<class T> inline
BinBaum<T>::~BinBaum() {
	loesche(wurzel);
}

template<class T>
void BinBaum<T>::neuesBlatt(Element*& bZgr, const T& x, Element* v) {
	if (bZgr == 0)
		bZgr = new Element(x, v);
	else
		if (x < bZgr->schluessel)              // Kleiner?
			neuesBlatt(bZgr->links, x, bZgr);    // Dann nach links,
		else                                   // sonst
			neuesBlatt(bZgr->rechts, x, bZgr);   // nach rechts.
}

template<class T> inline
void BinBaum<T>::neuesBlatt(const T& x)  {
	neuesBlatt(wurzel, x);
}

template<class T> inline
typename BinBaum<T>::Iter BinBaum<T>::start() {
	return const_cast<Element*>(erster());
}

template<class T> inline
typename BinBaum<T>::Iter BinBaum<T>::ende() {
	return 0;
}

template<class T> inline
typename BinBaum<T>::Iter BinBaum<T>::suchen(const T& x) {
	return const_cast<Element*>(suche(x));
}

template<class T>
void BinBaum<T>::drucke(std::ostream& os, const Element* z, int i) const {
	if (z) {
		drucke(os, z->links, i+3);         // erst links
		os << '\n' << std::setw(i) << z->schluessel;  // dann mitte
		drucke(os, z->rechts, i+3);        // und rechts
	}
}

template<class T> inline
std::ostream& BinBaum<T>::drucke(std::ostream& os) const {
	drucke(os, wurzel, 3);
	return os;
}

template<class T> inline
std::ostream& operator<<(std::ostream& os, const BinBaum<T>& b) {
	return b.drucke(os);
}

#endif
